use crate::{entity::EntityVisitor, schema::AssocData, DBResult};

// ----------------------------------------------------------------------
// Datum and related types
// ----------------------------------------------------------------------

/// Represents a data field in an Entity.
pub trait Datum: 'static {
    fn sql_type() -> &'static str;

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize);
    fn build_from<'a>(
        adata: AssocData,
        stmt: &mut sqlite::Statement<'a>,
        index: usize,
    ) -> DBResult<(Self, usize)>
    where
        Self: Sized;

    fn accept_entity_visitor(_: &mut impl EntityVisitor) {}
}

/// A fixed-length list of EntityDatums, usually a tuple.
pub trait DatumList {
    fn accept(&self, visitor: &mut impl DatumVisitor);
}

/// A walker for a EntityDatumList instance.
pub trait DatumVisitor {
    fn visit<ED: Datum>(&mut self, datum: &ED);
}

// trait implementations for EntityDatumList
mod datum_list;
